#! /usr/bin/env python

from .errors import NoCallBack
import aiohttp
import json

class Promise:
	def __init__(self, prev_promise, then_callable=None, catch_callable=None, finally_callable=None):
		self.prev_promise = prev_promise
		# Basic Variables
		self.data = None
		self.resolved = False
		self.next_promise = None

		# Add a check here to see if atleast one of the callables are provided
		if not then_callable and not catch_callable and not finally_callable:
			raise NoCallBack("Callback is needed to form a chain")
		self.then_callable = then_callable
		self.catch_callable = catch_callable
		self.finally_callable = finally_callable
		
	# Basic Methods for user
	def then(self, callable):
		self.next_promise = Promise(self, then_callable=callable)
		return self.next_promise
	
	def catch(self, callable):
		self.next_promise = Promise(self, catch_callable=callable)
		return self.next_promise
	
	def finally_(self, callable):
		self.next_promise = Promise(self, finally_callable=callable)
		return self.next_promise
	
	# Util methods for user
	def toJSON(self):
		return self.then(lambda data: json.loads(data))
	
	# Methods not for user
	def resolve(self, value):
		self.resolved = True
		try:
			# Give More Data to the callable
			self.data = self.then_callable(value)
			self.next_promise.resolve(self.data)
		except Exception as err:
			if self.catch_callable:
				self.catch_callable(err)
			elif self.next_promise:
				self.next_promise.rejected(value)
			else:
				raise err
		finally:
			if self.finally_callable:
				self.finally_callable()
	
	# Methods for working with the class
	def __str__(self):
		if self.resolved:
			return "Promise {<fulfilled>: {0}}".format(self.data)
		else:
			return "Promise {<pending>}"
	
	def __bool__(self):
		return self.resolved
	
	# If the user is using any other method which is not from class then return the data
	def __getattr__(self, name):
		return getattr(name, self.data)
	
	def __getitem__(self, key):
		return self.data[key]


async def fetch(url):
	empt_promise = Promise(None)
	yield empt_promise
	async with aiohttp.ClientSession() as session:
		async with session.get(url) as response:
			empt_promise.resolve(await response)


class NoCallBack:
	pass

class ResponseError:
	def __init__(self, status_code:int, response, expected_status_code:int=200):
		self.status_code = status_code
		self.response = response
		self.expected_status_code = expected_status_code
	
	def __str__(self) -> str:
		return f"The server responded with a {self.status_code} status code." + ("" if self.expected_status_code == 200 else f" Expected {self.expected_status_code}")


#! /usr/bin/env python

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

requires = [
	"aiohttp==3.8.1",
]

setuptools.setup(
    name="JSFetch",
    version="0.0.6",
    author="FSChatBot",
    description="Lets you make fetch requests in python as you would in JavaScript",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fschatbot/JSFetch",
    project_urls={
        "Bug Tracker": "https://github.com/fschatbot/JSFetch/issues",
    },
    classifiers=[
		"Development Status :: 3 - Alpha",
		"Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
	keywords = "Networking, Javascript, JS, Fetch, Async, AsyncIO, AsyncHTTPClient, AsyncHTTP",
    package_dir={"JSFetch": "src"},
	packages=["JSFetch"],
	install_requires=requires,
    python_requires=">=3.10",
)